-- $Id: Pascal.l,v 2.2 1997/09/20 17:48:31 nestor Exp $
-- $Locker:  $
-- $Log: Pascal.l,v $
-- Revision 2.2  1997/09/20 17:48:31  nestor
-- Correct two problems with comments: remove the original comment
-- delimiters in Ada comments, admit odd number of stars between
-- delimiters in comments of this kind (*****).
--
-- Revision 2.1  1997/08/23  17:16:27  nestor
-- Laurent Gasser's correction for MacOS extended Pascal
-- added tokens: unit, interface, implementation, uses
-- remove reserved keywords in upper-case, use case insensitivity
-- remove definitions of LT [<] and GT [>]
-- add tokens @ and &
-- add hexadecimal notation
--
-- Revision 1.1  1997/08/22  21:00:01  nestor
-- Martin C. Carlisle's original version, standard Pascal
--
-- aflex specification for Pascal
--
-- Martin C. Carlisle, US Air Force Academy
-- carlislem@acm.org
-- http://www.usafa.af.mil/dfcs/bios/carlisle.html
-- November 26, 1996

-- Laurent Gasser, Hirondelle Lumineuse
-- lga@sma.ch
-- June 15, 1997
-- Adaptation to extended MacOS Pascal
--
-- added tokens: unit, interface, implementation, uses
-- remove reserved keywords in upper-case, use case insensitivity
-- remove definitions of LT [<] and GT [>]
-- add tokens @ and &
-- add hexadecimal notation
--
-- usage: aflex -i Pascal.l

LETTER [a-zA-Z]
DIGIT [0-9]
HEXA [0-9a-fA-F]
ALPHANUM [0-9a-zA-Z]
IDCHAR [0-9a-zA-Z_]
PERIOD [.]
STAR [*]
PLUS [+]
SLASH [/]
LPAREN [(]
RPAREN [)]
LBRACK \[
RBRACK \]
LBRACE [{]
RBRACE [}]
NOTRPARENSTAR [^)*]
NOTSTAR [^*]
NOTRBRACE [^}]
NOTTICK [^']
UPARROW \^ 

%%

-- Pascal comments
{LPAREN}{STAR}(({NOTSTAR}*)|({STAR}{NOTRPARENSTAR}))*({STAR}+{RPAREN}) {
         PascalHelp.Comment(YYText(3..YYLength-2));
         }
{LBRACE}{NOTRBRACE}*{RBRACE} {PascalHelp.Comment(YYText(2..YYLength-1));}

-- Regular pascal stuff
"&" {return AMPERSAND_t;}
"@" {return AT_t;}
{LBRACK} {return LBRACK_t;}
{RBRACK} {return RBRACK_t;}
, { return COMMA_t; }
{UPARROW} { return UPARROW_t; }
{PERIOD}{PERIOD} { return DOUBLEDOT_t; }
{PERIOD} { return PERIOD_t; }
; { return SEMICOLON_t; }
:= {return ASSIGN_t;}
= {return EQUAL_t;}
: { return COLON_t; }
-> { return ARROW_t; }
{PLUS} { return PLUS_t; }
- { return MINUS_t; }
"<=" { return LE_t; }
">=" { return GE_t; }
"<>" { return NE_t; }
"<" { return LT_t; }
">" { return GT_t; }
{STAR}{STAR} { return DOUBLESTAR_t; }
{STAR} { return TIMES_t; }
{SLASH} { return DIVIDE_t; }
div {return DIV_t; }
mod {return MOD_t; }
{LPAREN} {return LPAREN_t;}
{RPAREN} {return RPAREN_t;}
{LBRACK} {return LBRACK_t;}
{RBRACK} {return RBRACK_t;}
and {return AND_t;}
or {return OR_t;}
not {return NOT_t;}
begin { return BEGIN_t; }
nil { return NIL_t; }
in { return IN_t; }
to { return TO_t; }
program { return PROGRAM_t; }
external { return EXTERNAL_t; }
otherwise { return OTHERWISE_t; }
end { return END_t; }
with { return WITH_t; }
label { return LABEL_t; }
const { return CONST_t; }
type { return TYPE_t; }
var { return VAR_t; }
array { return ARRAY_t; }
record { return RECORD_t; }
packed { return PACKED_t; }
of { return OF_t; }
case { return CASE_t; }
set { return SET_t; }
file { return FILE_t; }
forward { return FORWARD_t; }
procedure { return PROCEDURE_t; }
function { return FUNCTION_t; }
goto { return GOTO_t; }
if { return IF_t; }
for { return FOR_t; }
while { return WHILE_t; }
repeat { return REPEAT_t; }
until { return UNTIL_t; }
do { return DO_t; }
then { return THEN_t; }
else { return ELSE_t; }
downto { return DOWNTO_t; }
write { return WRITE_t; }
writeln { return WRITELN_t; }
char {return CHAR_t;}
real {return REAL_t;}
true {return TRUE_t;}
false {return FALSE_t;}
unit {return UNIT_t;}
uses {return USES_t;}
interface {return INTERFACE_t;}
implementation {return IMPLEMENTATION_t;}
{LETTER}{IDCHAR}* {return ID_t;}
{DIGIT}+{PERIOD}{DIGIT}+|{DIGIT}+ {return CONSTANT_t;}
"$"{HEXA}+ {return HEXADECIMAL_t;}
'{NOTTICK}' {return CHAR_CONST_t;}
'({NOTTICK}('')?)*' {return STRING_t;}

%%

WITH Pascal_Tokens; 
WITH PascalHelp;
USE Pascal_Tokens;

##
