-- $Id: yyparse.adb,v 2.1 1997/08/24 07:58:34 nestor Rel $
-- $Locker:  $
-- $Log: yyparse.adb,v $
-- Revision 2.1  1997/08/24 07:58:34  nestor
-- Laurent Gasser's correction for MacOS extended Pascal
--
-- Revision 1.1  1997/08/23  08:48:14  nestor
-- Martin C. Carlisle's original version, standard Pascal
--
-- p2ada
-- a source converter from Pascal MacOS to Ada
-- original version Martin C. Carlisle (November 1996)
-- extended for MacOS by Laurent Gasser (June 1997)

with Pascal_Tokens, Pascal_Shift_Reduce, Pascal_Goto;
use  Pascal_Tokens, Pascal_Shift_Reduce, Pascal_Goto;
with Pascal_DFA, YYroutines, Text_IO, PascalHelp, YYerror; 
use  Pascal_DFA, YYroutines, Text_IO;

procedure YYParse is

   -- Rename User Defined Packages to Internal Names.
    package yy_goto_tables         renames
      Pascal_Goto;
    package yy_shift_reduce_tables renames
      Pascal_Shift_Reduce;
    package yy_tokens              renames
      Pascal_Tokens;

   use yy_tokens, yy_goto_tables, yy_shift_reduce_tables;

   procedure yyerrok;
   procedure yyclearin;

   package yy is

       -- the size of the value and state stacks
       stack_size : constant Natural := 300;

       -- subtype rule         is natural;
       subtype parse_state  is natural;
       -- subtype nonterminal  is integer;

       -- encryption constants
       default           : constant := -1;
       first_shift_entry : constant :=  0;
       accept_code       : constant := -1001;
       error_code        : constant := -1000;

       -- stack data used by the parser
       tos                : natural := 0;
       value_stack        : array(0..stack_size) of yy_tokens.yystype;
       state_stack        : array(0..stack_size) of parse_state;

       -- current input symbol and action the parser is on
       action             : integer;
       rule_id            : rule;
       input_symbol       : yy_tokens.token;


       -- error recovery flag
       error_flag : natural := 0;
          -- indicates  3 - (number of valid shifts after an error occurs)

       look_ahead : boolean := true;
       index      : integer;

       -- Is Debugging option on or off
        DEBUG : constant boolean := FALSE;

    end yy;


    function goto_state
      (state : yy.parse_state;
       sym   : nonterminal) return yy.parse_state;

    function parse_action
      (state : yy.parse_state;
       t     : yy_tokens.token) return integer;

    pragma inline(goto_state, parse_action);


    function goto_state(state : yy.parse_state;
                        sym   : nonterminal) return yy.parse_state is
        index : integer;
    begin
        index := goto_offset(state);
        while  integer(goto_matrix(index).nonterm) /= sym loop
            index := index + 1;
        end loop;
        return integer(goto_matrix(index).newstate);
    end goto_state;


    function parse_action(state : yy.parse_state;
                          t     : yy_tokens.token) return integer is
        index      : integer;
        tok_pos    : integer;
        default    : constant integer := -1;
    begin
        tok_pos := yy_tokens.token'pos(t);
        index   := shift_reduce_offset(state);
        while integer(shift_reduce_matrix(index).t) /= tok_pos and then
              integer(shift_reduce_matrix(index).t) /= default
        loop
            index := index + 1;
        end loop;
        return integer(shift_reduce_matrix(index).act);
    end parse_action;

-- error recovery stuff

    procedure handle_error is
      temp_action : integer;
    begin

      if yy.error_flag = 3 then -- no shift yet, clobber input.
      if yy.debug then
          put_line("Ayacc.YYParse: Error Recovery Clobbers " &
                   yy_tokens.token'image(yy.input_symbol));
      end if;
        if yy.input_symbol = yy_tokens.end_of_input then  -- don't discard,
        if yy.debug then
            put_line("Ayacc.YYParse: Can't discard END_OF_INPUT, quiting...");
        end if;
        raise yy_tokens.syntax_error;
        end if;

            yy.look_ahead := true;   -- get next token
        return;                  -- and try again...
    end if;

    if yy.error_flag = 0 then -- brand new error
        yyerror("Syntax Error");
    end if;

    yy.error_flag := 3;

    -- find state on stack where error is a valid shift --

    if yy.debug then
        put_line("Ayacc.YYParse: Looking for state with error as valid shift");
    end if;

    loop
        if yy.debug then
        put_line("Ayacc.YYParse: Examining State " &
              yy.parse_state'image(yy.state_stack(yy.tos)));
        end if;
        temp_action := parse_action(yy.state_stack(yy.tos), error);

            if temp_action >= yy.first_shift_entry then
                if yy.tos = yy.stack_size then
                    put_line(" Stack size exceeded on state_stack");
                    raise yy_Tokens.syntax_error;
                end if;
                yy.tos := yy.tos + 1;
                yy.state_stack(yy.tos) := temp_action;
                exit;
            end if;

        Decrement_Stack_Pointer :
        begin
          yy.tos := yy.tos - 1;
        exception
          when Constraint_Error =>
            yy.tos := 0;
        end Decrement_Stack_Pointer;

        if yy.tos = 0 then
          if yy.debug then
            put_line("Ayacc.YYParse: Error recovery popped entire stack, aborting...");
          end if;
          raise yy_tokens.syntax_error;
        end if;
    end loop;

    if yy.debug then
        put_line("Ayacc.YYParse: Shifted error token in state " &
              yy.parse_state'image(yy.state_stack(yy.tos)));
    end if;

    end handle_error;

   -- print debugging information for a shift operation
   procedure shift_debug(state_id: yy.parse_state; lexeme: yy_tokens.token) is
   begin
       put_line("Ayacc.YYParse: Shift "& yy.parse_state'image(state_id)&" on input symbol "&
               yy_tokens.token'image(lexeme) );
   end;

   -- print debugging information for a reduce operation
   procedure reduce_debug(rule_id: rule; state_id: yy.parse_state) is
   begin
       put_line("Ayacc.YYParse: Reduce by rule "&rule'image(rule_id)&" goto state "&
               yy.parse_state'image(state_id));
   end;

   -- make the parser believe that 3 valid shifts have occured.
   -- used for error recovery.
   procedure yyerrok is
   begin
       yy.error_flag := 0;
   end yyerrok;

   -- called to clear input symbol that caused an error.
   procedure yyclearin is
   begin
       -- yy.input_symbol := yylex;
       yy.look_ahead := true;
   end yyclearin;


begin
    -- initialize by pushing state 0 and getting the first input symbol
    yy.state_stack(yy.tos) := 0;

    loop

        yy.index := shift_reduce_offset(yy.state_stack(yy.tos));
        if integer(shift_reduce_matrix(yy.index).t) = yy.default then
            yy.action := integer(shift_reduce_matrix(yy.index).act);
        else
            if yy.look_ahead then
                yy.look_ahead   := false;
                yy.input_symbol := yylex;
            end if;
            yy.action :=
             parse_action(yy.state_stack(yy.tos), yy.input_symbol);
        end if;

        if yy.action >= yy.first_shift_entry then  -- SHIFT

            if yy.debug then
                shift_debug(yy.action, yy.input_symbol);
            end if;

            -- Enter new state
            if yy.tos = yy.stack_size then
                put_line(" Stack size exceeded on state_stack");
                raise yy_Tokens.syntax_error;
            end if;
            yy.tos := yy.tos + 1;
            yy.state_stack(yy.tos) := yy.action;
            yy.value_stack(yy.tos) := yylval;

        if yy.error_flag > 0 then  -- indicate a valid shift
            yy.error_flag := yy.error_flag - 1;
        end if;

            -- Advance lookahead
            yy.look_ahead := true;

        elsif yy.action = yy.error_code then       -- ERROR
            handle_error;

        elsif yy.action = yy.accept_code then
            if yy.debug then
                put_line("Ayacc.YYParse: Accepting Grammar...");
            end if;
            exit;

        else -- Reduce Action

            -- Convert action into a rule
            yy.rule_id  := -1 * yy.action;

            -- Execute User Action
            -- user_action(yy.rule_id);
            case yy.rule_id is

when  4 =>
--#line  133
  pascalhelp.package_name := 
yy.value_stack(yy.tos).text;
    pascalhelp.package_name_length := 
yy.value_stack(yy.tos).length;

when  6 =>
--#line  138
    
yyval.length := YYLength; 
    
yyval.text(1..YYLength) := YYText;

when  7 =>
--#line  143

    pascalhelp.put("PACKAGE ");
    pascalhelp.put(pascalhelp.package_name(1..pascalhelp.package_name_length));
    pascalhelp.put(" IS");
    

when  9 =>
--#line  156
    pascalhelp.put(";");

when  11 =>
--#line  158
    pascalhelp.put(";");

when  14 =>
--#line  164

    pascalhelp.put("END ");
    pascalhelp.put(pascalhelp.package_name(1..pascalhelp.package_name_length));
    pascalhelp.put_line(";");
    pascalhelp.put_line(" ");
    pascalhelp.put("PACKAGE BODY ");
    pascalhelp.put(pascalhelp.package_name(1..pascalhelp.package_name_length));
    pascalhelp.put(" IS");
    

when  15 =>
--#line  179

    pascalhelp.put("END ");
    pascalhelp.put(pascalhelp.package_name(1..pascalhelp.package_name_length));
    pascalhelp.put(";");
    

when  16 =>
--#line  186
pascalhelp.put("WITH ");

when  17 =>
--#line  188
pascalhelp.put(";");

when  19 =>
--#line  192
pascalhelp.put("PROCEDURE ");

when  20 =>
--#line  193
pascalhelp.put_line(" IS");

when  24 =>
--#line  205
pascalhelp.put_line(": Ada.Text_IO.File_Type;");

when  25 =>
--#line  207
pascalhelp.put_line(": Ada.Text_IO.File_Type;");

when  26 =>
--#line  210
pascalhelp.put(',');

when  35 =>
--#line  234
pascalhelp.Put("<<Label_"); pascalhelp.put(YYText); pascalhelp.put(">>");

when  40 =>
--#line  249
 pascalhelp.Put(": CONSTANT"); pascalhelp.DirectIO := False;

when  41 =>
--#line  251

     pascalhelp.DirectIO := True;
     IF 
yy.value_stack(yy.tos-1).vartype = boolean_type THEN
       pascalhelp.put("Boolean := ");
     ELSIF 
yy.value_stack(yy.tos-1).vartype = character_type THEN  
       pascalhelp.put("Character := ");
     ELSIF 
yy.value_stack(yy.tos-1).vartype = string_type THEN  
       pascalhelp.put("String := ");
     ELSE
       pascalhelp.put(":= ");
     END IF;
     pascalhelp.flush;
     pascalhelp.empty;
     pascalhelp.Put(";");
    

when  42 =>
--#line  268

yyval.vartype := 
yy.value_stack(yy.tos).vartype;

when  43 =>
--#line  269

yyval.vartype:=boolean_type;

when  44 =>
--#line  272

yyval.vartype := 
yy.value_stack(yy.tos).vartype;

when  45 =>
--#line  273

yyval.vartype := other_type;

when  47 =>
--#line  277
pascalhelp.put('+');

when  49 =>
--#line  278
pascalhelp.put('-');

when  51 =>
--#line  279
pascalhelp.put("OR");

when  53 =>
--#line  283

yyval.vartype := other_type;

when  59 =>
--#line  292
pascalhelp.put('(');

when  60 =>
--#line  294
pascalhelp.put(')'); 
yyval.vartype := 
yy.value_stack(yy.tos-1).vartype;

when  61 =>
--#line  295
pascalhelp.put("NOT");

when  63 =>
--#line  300

yyval.vartype := other_type;

when  64 =>
--#line  301
pascalhelp.put(YYText); 
yyval.vartype := character_type;

when  65 =>
--#line  302
pascalhelp.printstring(YYText); 
yyval.vartype := string_type;

when  66 =>
--#line  304
pascalhelp.put("NULL"); 
yyval.vartype := other_type;

when  67 =>
--#line  305
pascalhelp.put("True"); 
yyval.vartype := boolean_type;

when  68 =>
--#line  306
pascalhelp.put("False"); 
yyval.vartype := boolean_type;

when  69 =>
--#line  334

yyval.vartype := 
yy.value_stack(yy.tos).vartype;

when  70 =>
--#line  335

yyval.vartype := other_type;

when  71 =>
--#line  336
pascalhelp.put(YYText); 
yyval.vartype := character_type;

when  72 =>
--#line  337
PascalHelp.printstring(YYText); 
yyval.vartype := string_type;

when  73 =>
--#line  340
pascalhelp.put('+');

when  74 =>
--#line  341
pascalhelp.put('-');

when  75 =>
--#line  344
pascalhelp.put(YYText); 
yyval.vartype := other_type;

when  76 =>
--#line  346

       DECLARE
         the_number : string := YYText;
       BEGIN
         pascalhelp.Put("16#" & the_number(2..the_number'last) & "#");
         
yyval.vartype := other_type;
       END;
       

when  77 =>
--#line  354

yyval.vartype := other_type;

when  78 =>
--#line  355
pascalhelp.put(" True "); 
yyval.vartype := boolean_type;

when  79 =>
--#line  356
pascalhelp.put(" False "); 
yyval.vartype := boolean_type;

when  84 =>
--#line  371
pascalhelp.put("TYPE ");

when  85 =>
--#line  372
pascalhelp.put(" IS ");

when  86 =>
--#line  373
pascalhelp.put(';');

when  96 =>
--#line  391
pascalhelp.put('(');

when  97 =>
--#line  392
pascalhelp.put(')');

when  98 =>
--#line  395
pascalhelp.put("..");

when  106 =>
--#line  408
pascalhelp.put(" ARRAY (");

when  107 =>
--#line  410
pascalhelp.put(") OF ");

when  109 =>
--#line  414
pascalhelp.put(',');

when  116 =>
--#line  457
pascalhelp.put(" RECORD ");

when  117 =>
--#line  459

    if not pascalhelp.has_optional_semicolon then
      pascalhelp.put(';');
    end if;
    pascalhelp.has_optional_semicolon := FALSE;
    

when  118 =>
--#line  465
pascalhelp.put("END RECORD");

when  121 =>
--#line  472
pascalhelp.put(';');

when  126 =>
--#line  479
pascalhelp.has_optional_semicolon := TRUE;

when  127 =>
--#line  484
pascalhelp.put(':');

when  129 =>
--#line  489
pascalhelp.put("CASE");

when  130 =>
--#line  491
pascalhelp.put_line("IS");

when  131 =>
--#line  492
pascalhelp.put("WHEN ");

when  132 =>
--#line  493
pascalhelp.put("END CASE");

when  135 =>
--#line  501
pascalhelp.put("WHEN ");

when  138 =>
--#line  505
pascalhelp.put(" => ");

when  139 =>
--#line  507
pascalhelp.put(';');

when  141 =>
--#line  520
pascalhelp.put(" | ");

when  148 =>
--#line  533
pascalhelp.put("NEW Set(Of =>");

when  149 =>
--#line  534
pascalhelp.put(")");

when  151 =>
--#line  540
pascalhelp.put("Ada.Text_IO.File_Type");

when  152 =>
--#line  543
pascalhelp.Put("ACCESS ");

when  155 =>
--#line  548
pascalhelp.put("PROCEDURE");

when  156 =>
--#line  549
pascalhelp.put("PROCEDURE");

when  158 =>
--#line  552
pascalhelp.put("FUNCTION");

when  159 =>
--#line  553
pascalhelp.put(" IS ");

when  161 =>
--#line  554
pascalhelp.put("FUNCTION");

when  162 =>
--#line  556
pascalhelp.put(" IS ");

when  168 =>
--#line  574
    pascalhelp.put(':');

when  169 =>
--#line  576
    pascalhelp.put(";");

when  176 =>
--#line  598
DECLARE
         tok : Token;
       BEGIN
         tok := YYLex;
         IF (tok = FORWARD_t) OR (tok=EXTERNAL_t) THEN
           pascalhelp.put_line(";");
         ELSE
           pascalhelp.put_line(" IS");
         END IF;
         UnYYLex(tok);
      END;

when  182 =>
--#line  620
pascalhelp.put('(');

when  183 =>
--#line  621
pascalhelp.put(')');

when  184 =>
--#line  624
pascalhelp.put(';');

when  191 =>
--#line  634
pascalhelp.put(':');

when  193 =>
--#line  639
pascalhelp.put(": IN OUT ");

when  197 =>
--#line  646
pascalhelp.put("PROCEDURE");

when  200 =>
--#line  651
DECLARE
         tok : Token;
       BEGIN
         tok := YYLex;
         IF (tok = FORWARD_t) OR (tok=EXTERNAL_t) THEN
           pascalhelp.put_line(";");
         ELSE
           pascalhelp.put_line(" IS");
           pascalhelp.put("Result");
           pascalhelp.put(pascalhelp.FunctionName(1..pascalhelp.FnNameLength));
           pascalhelp.put(" : ");
           pascalhelp.put(
yy.value_stack(yy.tos-1).text(1..
yy.value_stack(yy.tos-1).length));
           pascalhelp.put_line(";");
           pascalhelp.put_line("BEGIN");
           pascalhelp.put("DECLARE ");
         END IF;
         UnYYLex(tok);
      END;

when  203 =>
--#line  675
pascalhelp.put("RETURN Result");
     pascalhelp.put(pascalhelp.FunctionName(1..pascalhelp.FnNameLength));
     pascalhelp.put_line(";");
     pascalhelp.put("END;");
     pascalhelp.FnNameLength := 0;

when  206 =>
--#line  687
pascalhelp.put("FUNCTION ");

when  207 =>
--#line  688
pascalhelp.FnNameLength := 
yy.value_stack(yy.tos).length; 
                pascalhelp.FunctionName(1..
yy.value_stack(yy.tos).length) := 
                  
yy.value_stack(yy.tos).text(1..
yy.value_stack(yy.tos).length);
               

when  208 =>
--#line  693
pascalhelp.put(" RETURN ");

when  209 =>
--#line  694

yyval.length := 
yy.value_stack(yy.tos).length;
     
yyval.text := 
yy.value_stack(yy.tos).text;
    

when  212 =>
--#line  704

yyval.length := 
yy.value_stack(yy.tos).length; 
yyval.text := 
yy.value_stack(yy.tos).text;

when  230 =>
--#line  739
pascalhelp.put("BEGIN");

when  231 =>
--#line  741
pascalhelp.put("END;");

when  234 =>
--#line  748
pascalhelp.put_line("LOOP");

when  235 =>
--#line  750
pascalhelp.put(" EXIT WHEN ");

when  236 =>
--#line  751
pascalhelp.put_line(";"); pascalhelp.put_line("END LOOP;");

when  237 =>
--#line  754
pascalhelp.put("WHILE ");

when  238 =>
--#line  755
pascalhelp.put_line(" LOOP");

when  239 =>
--#line  756
pascalhelp.put_line("END LOOP;");

when  240 =>
--#line  759
pascalhelp.put("FOR ");

when  241 =>
--#line  760
pascalhelp.put(" IN ");
              pascalhelp.DirectIO := False;
            

when  242 =>
--#line  763
pascalhelp.put("..");

when  243 =>
--#line  765
pascalhelp.put_line(" LOOP");

when  244 =>
--#line  767
pascalhelp.put_line("END LOOP;");

when  245 =>
--#line  771

    pascalhelp.put_line("--HELP!! WITH ");
    pascalhelp.put_line("DECLARE ");
    pascalhelp.put("  r : <type> RENAMES ");
    

when  246 =>
--#line  777
pascalhelp.put(';');

when  248 =>
--#line  782
pascalhelp.put("IF ");

when  249 =>
--#line  783
pascalhelp.put("THEN");

when  250 =>
--#line  785
pascalhelp.put("END IF;");

when  251 =>
--#line  788
pascalhelp.put("ELSE");

when  254 =>
--#line  793
pascalhelp.put(":=");

when  255 =>
--#line  795
pascalhelp.put(";");

when  256 =>
--#line  799
pascalhelp.put('(');

when  257 =>
--#line  801
pascalhelp.put(')');

when  261 =>
--#line  811
pascalhelp.put('.');

when  263 =>
--#line  815
pascalhelp.put(";");

when  264 =>
--#line  816
pascalhelp.put(';');

when  265 =>
--#line  817
pascalhelp.Put("Put");

when  266 =>
--#line  817
pascalhelp.put(";");

when  267 =>
--#line  818
pascalhelp.Put("Put");

when  268 =>
--#line  819
pascalhelp.put_line(";");
       pascalhelp.put("New_Line;");

when  271 =>
--#line  832
pascalhelp.put('(');

when  272 =>
--#line  833
pascalhelp.put(')');

when  274 =>
--#line  838
pascalhelp.put_line(");");
     pascalhelp.put("Put(");

when  278 =>
--#line  844
pascalhelp.put(',');

when  281 =>
--#line  848
pascalhelp.put(',');

when  282 =>
--#line  849
pascalhelp.put(",0");

when  283 =>
--#line  852
pascalhelp.put('(');

when  284 =>
--#line  854
pascalhelp.put(')');

when  285 =>
--#line  858
pascalhelp.put(',');

when  289 =>
--#line  868
pascalhelp.put("GOTO ");

when  290 =>
--#line  869
pascalhelp.put_line(";");

when  291 =>
--#line  872
pascalhelp.put("CASE ");

when  292 =>
--#line  873
pascalhelp.put_line(" IS ");

when  293 =>
--#line  874
pascalhelp.put("WHEN ");

when  294 =>
--#line  875
pascalhelp.put_line("END CASE;");

when  301 =>
--#line  892
pascalhelp.put("WHEN ");

when  304 =>
--#line  896
pascalhelp.put("=>");

when  306 =>
--#line  900
pascalhelp.put("WHEN OTHERS =>");

when  308 =>
--#line  901
pascalhelp.put("WHEN OTHERS =>");

when  312 =>
--#line  909
 pascalhelp.DirectIO := True;
        pascalhelp.flush;
        pascalhelp.empty;
      

when  313 =>
--#line  914
 pascalhelp.DirectIO := True;
        pascalhelp.put(" REVERSE ");
        pascalhelp.flush;
        pascalhelp.empty;
      

when  326 =>
--#line  954
pascalhelp.put("System.Address_To_Access_Conversions.To_Address(");

when  327 =>
--#line  955
pascalhelp.put(')');

when  331 =>
--#line  959
pascalhelp.put('(');

when  332 =>
--#line  961
pascalhelp.put(')');

when  333 =>
--#line  962
 pascalhelp.put("NOT");

when  340 =>
--#line  976
pascalhelp.put('.');

when  342 =>
--#line  978
pascalhelp.put('(');

when  343 =>
--#line  980
pascalhelp.put(')');

when  344 =>
--#line  984
pascalhelp.put(".all");

when  345 =>
--#line  995
pascalhelp.Put(YYText);

when  346 =>
--#line  997

         DECLARE
           the_number : string := YYText;
         BEGIN
           pascalhelp.Put("16#" & the_number(2..the_number'last) & "#");
         END;
         

when  351 =>
--#line  1014
pascalhelp.put("..");

when  354 =>
--#line  1019
pascalhelp.put('+');

when  355 =>
--#line  1020
pascalhelp.put('-');

when  356 =>
--#line  1021
pascalhelp.put(" OR ");

when  357 =>
--#line  1024
pascalhelp.put('*');

when  358 =>
--#line  1025
pascalhelp.put('/');

when  359 =>
--#line  1026
pascalhelp.put(" DIV ");

when  360 =>
--#line  1027
pascalhelp.put(" MOD ");

when  361 =>
--#line  1028
pascalhelp.put(" AND ");

when  362 =>
--#line  1029
pascalhelp.put('&');

when  363 =>
--#line  1030
pascalhelp.put("**");

when  364 =>
--#line  1033
pascalhelp.put('=');

when  365 =>
--#line  1034
pascalhelp.put("/=");

when  366 =>
--#line  1035
pascalhelp.put('<');

when  367 =>
--#line  1036
pascalhelp.put('>');

when  368 =>
--#line  1037
pascalhelp.put("<=");

when  369 =>
--#line  1038
pascalhelp.put(">=");

when  370 =>
--#line  1039
pascalhelp.put(" IN ");

when  371 =>
--#line  1042
IF YYLength = pascalhelp.FnNameLength AND THEN
                      YYtext(1..YYLength) = 
                      pascalhelp.FunctionName(1..YYLength) THEN
                     pascalhelp.put("Result");
                   END IF;
                   
yyval.length:=YYLength; 
yyval.text(1..YYLength):=YYText;
                   pascalhelp.put(YYText);
                  

when  372 =>
--#line  1050

yyval.length:=9; 
yyval.text(1..9):="Character";
              pascalhelp.put(" Character "); 

when  373 =>
--#line  1052

yyval.length:=5; 
yyval.text(1..5):="Float";
              pascalhelp.put("Float"); 

                when others => null;
            end case;

            -- Pop RHS states and goto next state
            yy.tos      := yy.tos - rule_length(yy.rule_id) + 1;
            if yy.tos > yy.stack_size then
                put_line(" Stack size exceeded on state_stack");
                raise yy_Tokens.syntax_error;
            end if;
            yy.state_stack(yy.tos) := goto_state(yy.state_stack(yy.tos-1) ,
                                 get_lhs_rule(yy.rule_id));

            yy.value_stack(yy.tos) := yyval;

            if yy.debug then
                reduce_debug(yy.rule_id,
                    goto_state(yy.state_stack(yy.tos - 1),
                               get_lhs_rule(yy.rule_id)));
            end if;

        end if;
    end loop;

end yyparse;
